From e4aba403ae9362e84b83b8c4a0866d8e4bbf45fc Mon Sep 17 00:00:00 2001
From: Pavel Baturko <pabftk@gmail.com>
Date: Sun, 20 Jan 2013 23:26:28 +0000
Subject: [PATCH] Add decoding UCS2 to UTF8, Extend gsm338_get_sms_alphabet to support more DCSs, Added function to check availability of UFT-8

---
 src/gsm/gsm0411_utils.c |    3 +-
 src/utils.c             |   65 +++++++++++++++++++++++++++++++++++++++++++++++
 2 files changed, 67 insertions(+), 1 deletions(-)

diff --git a/src/gsm/gsm0411_utils.c b/src/gsm/gsm0411_utils.c
index fe69bf4..d6c4482 100644
--- a/src/gsm/gsm0411_utils.c
+++ b/src/gsm/gsm0411_utils.c
@@ -231,7 +231,7 @@ enum sms_alphabet gsm338_get_sms_alphabet(uint8_t dcs)
 	uint8_t cgbits = dcs >> 4;
 	enum sms_alphabet alpha = DCS_NONE;
 
-	if ((cgbits & 0xc) == 0) {
+	if ((cgbits & 0x08) == 0) {
 		if (cgbits & 2) {
 			LOGP(DLSMS, LOGL_NOTICE,
 			     "Compressed SMS not supported yet\n");
@@ -240,6 +240,7 @@ enum sms_alphabet gsm338_get_sms_alphabet(uint8_t dcs)
 
 		switch ((dcs >> 2)&0x03) {
 		case 0:
+		case 3:
 			alpha = DCS_7BIT_DEFAULT;
 			break;
 		case 1:
diff --git a/src/utils.c b/src/utils.c
index c36979c..1634c23 100644
--- a/src/utils.c
+++ b/src/utils.c
@@ -3,6 +3,7 @@
 #include <stdint.h>
 #include <errno.h>
 #include <stdio.h>
+#include <locale.h>
 
 #include <osmocom/core/utils.h>
 
@@ -220,4 +221,68 @@ void osmo_str2upper(char *out, const char *in)
 }
 #endif /* HAVE_CTYPE_H */
 
+/*! \brief Convert UCS2 string to UTF8 string.
+ *  \param[out] decoded pointer to buffer which will contain decoded string
+ *  \param[in] decoded_buf_length length of 'decoded' buffer
+ *  \param[in] user_data pointer to buffer which contains UCS2 string
+ *  \param[in] length length of 'user_data' buffer
+ *  \returns length of decoded string
+ *
+ * This function will convert UCS2 string to UTF-8 string. 
+ * Taken from Ben Bullock (lemoda.net).
+ */
+uint16_t osmo_ucs2_to_utf8_decode(char *decoded, const uint16_t decoded_buf_length, const uint8_t *user_data, const uint8_t length)
+{
+	uint16_t i, j = 0;
+
+	for (i = 0; i < length; i++) {
+		uint16_t ucs2;
+
+		ucs2 = ((user_data[2 * i] << 8) | user_data[2 * i + 1]);
+		if (ucs2 < 0x80) {
+			if (j >= decoded_buf_length-1)
+				break;
+			decoded[j++] = ucs2;
+		} else if (ucs2 >= 0x80  && ucs2 < 0x800) {
+			if (j >= decoded_buf_length-2)
+				break;
+			decoded[j++] = (ucs2 >> 6)   | 0xC0;
+			decoded[j++] = (ucs2 & 0x3F) | 0x80;
+		} else if (ucs2 >= 0x800 && ucs2 < 0xFFFF) {
+			if (j >= decoded_buf_length-3)
+				break;
+			decoded[j++] = ((ucs2 >> 12)       ) | 0xE0;
+			decoded[j++] = ((ucs2 >> 6 ) & 0x3F) | 0x80;
+			decoded[j++] = ((ucs2      ) & 0x3F) | 0x80;
+		}
+	}
+	decoded[j] = '\0';
+
+	return j;
+}
+
+/*! \brief Check if terminal supports UFT-8 locale
+ *  \returns 1 if terminal supports UTF-8, 0 - otherwise
+ *
+ * This function will set default locale  
+ * and check if it is 'utf-8' locale. Then it will 
+ * restore original locale.
+ */
+uint8_t osmo_check_utf8_locale()
+{
+	char *curr_locale, *old_locale;
+	uint8_t have_utf8_locale;
+	uint16_t i;
+
+	old_locale = setlocale(LC_ALL, NULL);
+	curr_locale = setlocale(LC_ALL, "");
+	for (i = 0; i < strlen(curr_locale); i++)
+		curr_locale[i] = tolower(curr_locale[i]);
+	have_utf8_locale = ((strstr(curr_locale, "utf-8") != NULL) || 
+		(strstr(curr_locale, "utf8" ) != NULL)) ? 1 : 0;
+	setlocale(LC_ALL, old_locale);
+
+	return have_utf8_locale;
+}
+
 /*! @} */
-- 
1.7.2.5

